@@ -1,16 +1,24 @@
+# geoms
 from .geom_abline import geom_abline
 from .geom_area import geom_area
 from .geom_bar import geom_bar
 from .geom_density import geom_density
-from .geom_hist import geom_hist
+from .geom_errorbar import geom_errorbar
+from .geom_histogram import geom_histogram
 from .geom_hline import geom_hline
 from .geom_jitter import geom_jitter
 from .geom_line import geom_line
+from .geom_now_its_art import geom_now_its_art
 from .geom_point import geom_point
+from .geom_step import geom_step
+from .geom_text import geom_text
+from .geom_tile import geom_tile
 from .geom_vline import geom_vline
-from .facet_grid import facet_grid
-from .facet_wrap import facet_wrap
+# stats
 from .stat_bin2d import stat_bin2d
 from .stat_smooth import stat_smooth
+# misc
+from .facet_grid import facet_grid
+from .facet_wrap import facet_wrap
 from .chart_components import *
 
